@@ -17,7 +17,7 @@
                               // avoid retaining a reference to the bootstrap
                               // object.
                               { _setupTraceCategoryState,
-                                _setupProcessObject, _setupNextTick,
+                                _setupNextTick,
                                 _setupPromises, _chdir, _cpuUsage,
                                 _hrtime, _hrtimeBigInt,
                                 _memoryUsage, _rawDebug,
@@ -30,13 +30,6 @@
   const isMainThread = internalBinding('worker').threadId === 0;
 
   function startup() {
-    const EventEmitter = NativeModule.require('events');
-
-    const origProcProto = Object.getPrototypeOf(process);
-    Object.setPrototypeOf(origProcProto, EventEmitter.prototype);
-
-    EventEmitter.call(process);
-
     setupTraceCategoryState();
 
     setupProcessObject();
@@ -103,21 +96,20 @@
       NativeModule.require('internal/process/write-coverage').setup();
 
     if (process.env.NODE_V8_COVERAGE) {
-      const { resolve } = NativeModule.require('path');
-      process.env.NODE_V8_COVERAGE = resolve(process.env.NODE_V8_COVERAGE);
-      NativeModule.require('internal/process/coverage').setup();
+      NativeModule.require('internal/process/coverage').setupExitHooks();
     }
 
     if (process.config.variables.v8_enable_inspector) {
       NativeModule.require('internal/inspector_async_hook').setup();
     }
 
-    const { getOptions } = internalBinding('options');
-    const helpOption = getOptions('--help');
-    const completionBashOption = getOptions('--completion-bash');
-    const experimentalModulesOption = getOptions('--experimental-modules');
-    const experimentalVMModulesOption = getOptions('--experimental-vm-modules');
-    const experimentalWorkerOption = getOptions('--experimental-worker');
+    const { getOptionValue } = NativeModule.require('internal/options');
+    const helpOption = getOptionValue('--help');
+    const completionBashOption = getOptionValue('--completion-bash');
+    const experimentalModulesOption = getOptionValue('--experimental-modules');
+    const experimentalVMModulesOption =
+      getOptionValue('--experimental-vm-modules');
+    const experimentalWorkerOption = getOptionValue('--experimental-worker');
     if (helpOption) {
       NativeModule.require('internal/print_help').print(process.stdout);
       return;
@@ -380,53 +372,10 @@
   }
 
   function setupProcessObject() {
-    _setupProcessObject(pushValueToArray);
-
-    function pushValueToArray() {
-      for (var i = 0; i < arguments.length; i++)
-        this.push(arguments[i]);
-    }
-
-    // Deprecated specific process.binding() modules, but not all, allow
-    // selective fallback to internalBinding for the deprecated ones.
-    const { SafeSet } = NativeModule.require('internal/safe_globals');
-    const processBinding = process.binding;
-    // internalBindingWhitelist contains the name of internalBinding modules
-    // that are whitelisted for access via process.binding()... this is used
-    // to provide a transition path for modules that are being moved over to
-    // internalBinding.
-    const internalBindingWhitelist =
-      new SafeSet([
-        'cares_wrap',
-        'fs_event_wrap',
-        'icu',
-        'udp_wrap',
-        'uv',
-        'pipe_wrap',
-        'http_parser',
-        'process_wrap',
-        'v8',
-        'tty_wrap',
-        'stream_wrap',
-        'signal_wrap',
-        'crypto',
-        'contextify',
-        'tcp_wrap',
-        'tls_wrap',
-        'util',
-        'async_wrap',
-        'url',
-        'spawn_sync',
-        'js_stream',
-        'zlib',
-        'buffer',
-        'natives',
-        'constants']);
-    process.binding = function binding(name) {
-      return internalBindingWhitelist.has(name) ?
-        internalBinding(name) :
-        processBinding(name);
-    };
+    const EventEmitter = NativeModule.require('events');
+    const origProcProto = Object.getPrototypeOf(process);
+    Object.setPrototypeOf(origProcProto, EventEmitter.prototype);
+    EventEmitter.call(process);
   }
 
   function setupGlobalVariables() {
@@ -687,7 +636,7 @@
   function tryGetCwd(path) {
     try {
       return process.cwd();
-    } catch (ex) {
+    } catch {
       // getcwd(3) can fail if the current working directory has been deleted.
       // Fall back to the directory name of the (absolute) executable path.
       // It's not really correct but what are the alternatives?
@@ -766,10 +715,9 @@
 
     const get = () => {
       const {
-        getOptions,
         envSettings: { kAllowedInEnvironment }
       } = internalBinding('options');
-      const { options, aliases } = getOptions();
+      const { options, aliases } = NativeModule.require('internal/options');
 
       const allowedNodeEnvironmentFlags = [];
       for (const [name, info] of options) {
