@@ -38,8 +38,6 @@ const { owner_symbol } = require('internal/async_hooks').symbols;
 const { convertToValidSignal } = require('internal/util');
 const { isArrayBufferView } = require('internal/util/types');
 const spawn_sync = internalBinding('spawn_sync');
-const { HTTPParser } = internalBinding('http_parser');
-const { freeParser } = require('_http_common');
 const { kStateSymbol } = require('internal/dgram');
 
 const {
@@ -57,6 +55,10 @@ const { SocketListSend, SocketListReceive } = SocketList;
 
 // Lazy loaded for startup performance.
 let StringDecoder;
+// Lazy loaded for startup performance and to allow monkey patching of
+// internalBinding('http_parser').HTTPParser.
+let freeParser;
+let HTTPParser;
 
 const MAX_HANDLE_RETRANSMISSIONS = 3;
 
@@ -121,6 +123,12 @@ const handleConversion = {
         handle.onread = nop;
         socket._handle = null;
         socket.setTimeout(0);
+
+        if (freeParser === undefined)
+          freeParser = require('_http_common').freeParser;
+        if (HTTPParser === undefined)
+          HTTPParser = internalBinding('http_parser').HTTPParser;
+
         // In case of an HTTP connection socket, release the associated
         // resources
         if (socket.parser && socket.parser instanceof HTTPParser) {
