@@ -74,7 +74,7 @@ function getUIntOption(options, key) {
 function createCipherBase(cipher, credential, options, decipher, iv) {
   const authTagLength = getUIntOption(options, 'authTagLength');
 
-  legacyNativeHandle(this, new CipherBase(decipher));
+  this[kHandle] = new CipherBase(decipher);
   if (iv === undefined) {
     this[kHandle].init(cipher, credential, authTagLength);
   } else {
@@ -219,6 +219,8 @@ Cipher.prototype.setAAD = function setAAD(aadbuf, options) {
   return this;
 };
 
+legacyNativeHandle(Cipher);
+
 function Cipheriv(cipher, key, iv, options) {
   if (!(this instanceof Cipheriv))
     return new Cipheriv(cipher, key, iv, options);
@@ -254,6 +256,7 @@ function addCipherPrototypeFunctions(constructor) {
 
 inherits(Cipheriv, LazyTransform);
 addCipherPrototypeFunctions(Cipheriv);
+legacyNativeHandle(Cipheriv);
 
 function Decipher(cipher, password, options) {
   if (!(this instanceof Decipher))
@@ -264,6 +267,7 @@ function Decipher(cipher, password, options) {
 
 inherits(Decipher, LazyTransform);
 addCipherPrototypeFunctions(Decipher);
+legacyNativeHandle(Decipher);
 
 
 function Decipheriv(cipher, key, iv, options) {
@@ -275,6 +279,7 @@ function Decipheriv(cipher, key, iv, options) {
 
 inherits(Decipheriv, LazyTransform);
 addCipherPrototypeFunctions(Decipheriv);
+legacyNativeHandle(Decipheriv);
 
 module.exports = {
   Cipher,
