@@ -2,7 +2,7 @@
 
 const Stream = require('stream');
 const Readable = Stream.Readable;
-const binding = process.binding('http2');
+const binding = internalBinding('http2');
 const constants = binding.constants;
 const {
   ERR_HTTP2_HEADERS_SENT,
@@ -241,6 +241,13 @@ function onStreamCloseRequest() {
   req.emit('close');
 }
 
+function onStreamTimeout(kind) {
+  return function onStreamTimeout() {
+    const obj = this[kind];
+    obj.emit('timeout');
+  };
+}
+
 class Http2ServerRequest extends Readable {
   constructor(stream, headers, options, rawHeaders) {
     super(options);
@@ -263,6 +270,7 @@ class Http2ServerRequest extends Readable {
     stream.on('error', onStreamError);
     stream.on('aborted', onStreamAbortedRequest);
     stream.on('close', onStreamCloseRequest);
+    stream.on('timeout', onStreamTimeout(kRequest));
     this.on('pause', onRequestPause);
     this.on('resume', onRequestResume);
   }
@@ -416,6 +424,7 @@ class Http2ServerResponse extends Stream {
     stream.on('aborted', onStreamAbortedResponse);
     stream.on('close', onStreamCloseResponse);
     stream.on('wantTrailers', onStreamTrailersReady);
+    stream.on('timeout', onStreamTimeout(kResponse));
   }
 
   // User land modules such as finalhandler just check truthiness of this
