@@ -31,14 +31,16 @@ CSI.kClearLine = CSI`2K`;
 CSI.kClearScreenDown = CSI`0J`;
 
 if (process.binding('config').hasIntl) {
-  const icu = process.binding('icu');
+  const icu = internalBinding('icu');
   getStringWidth = function getStringWidth(str, options) {
     options = options || {};
     if (!Number.isInteger(str))
       str = stripVTControlCharacters(String(str));
-    return icu.getStringWidth(str,
-                              Boolean(options.ambiguousAsFullWidth),
-                              Boolean(options.expandEmojiSequence));
+    return icu.getStringWidth(
+      str,
+      Boolean(options.ambiguousAsFullWidth),
+      Boolean(options.expandEmojiSequence)
+    );
   };
   isFullWidthCodePoint =
     function isFullWidthCodePoint(code, options) {
