@@ -245,7 +245,14 @@ function onParseError(flags, input) {
 // Reused by URL constructor and URL#href setter.
 function parse(url, input, base) {
   const base_context = base ? base[context] : undefined;
-  url[context] = new URLContext();
+  // In the URL#href setter
+  if (!url[context]) {
+    Object.defineProperty(url, context, {
+      enumerable: false,
+      configurable: false,
+      value: new URLContext()
+    });
+  }
   _parse(input.trim(), -1, base_context, undefined,
          onParseComplete.bind(url), onParseError);
 }
@@ -1437,7 +1444,11 @@ function toPathIfFileURL(fileURLOrPath) {
 }
 
 function NativeURL(ctx) {
-  this[context] = ctx;
+  Object.defineProperty(this, context, {
+    enumerable: false,
+    configurable: false,
+    value: ctx
+  });
 }
 NativeURL.prototype = URL.prototype;
 
