@@ -6,18 +6,23 @@ namespace util {
 
 using v8::ALL_PROPERTIES;
 using v8::Array;
+using v8::Boolean;
 using v8::Context;
 using v8::Function;
 using v8::FunctionCallbackInfo;
+using v8::IndexFilter;
 using v8::Integer;
 using v8::Isolate;
+using v8::KeyCollectionMode;
 using v8::Local;
+using v8::NewStringType;
 using v8::Object;
 using v8::ONLY_CONFIGURABLE;
 using v8::ONLY_ENUMERABLE;
 using v8::ONLY_WRITABLE;
 using v8::Private;
 using v8::Promise;
+using v8::PropertyFilter;
 using v8::Proxy;
 using v8::SKIP_STRINGS;
 using v8::SKIP_SYMBOLS;
@@ -37,13 +42,13 @@ static void GetOwnNonIndexProperties(
 
   Local<Array> properties;
 
-  v8::PropertyFilter filter =
-    static_cast<v8::PropertyFilter>(args[1].As<Uint32>()->Value());
+  PropertyFilter filter =
+    static_cast<PropertyFilter>(args[1].As<Uint32>()->Value());
 
   if (!object->GetPropertyNames(
-        context, v8::KeyCollectionMode::kOwnOnly,
+        context, KeyCollectionMode::kOwnOnly,
         filter,
-        v8::IndexFilter::kSkipIndices)
+        IndexFilter::kSkipIndices)
           .ToLocal(&properties)) {
     return;
   }
@@ -51,6 +56,7 @@ static void GetOwnNonIndexProperties(
 }
 
 static void GetPromiseDetails(const FunctionCallbackInfo<Value>& args) {
+  Environment* env = Environment::GetCurrent(args);
   // Return undefined if it's not a Promise.
   if (!args[0]->IsPromise())
     return;
@@ -61,14 +67,15 @@ static void GetPromiseDetails(const FunctionCallbackInfo<Value>& args) {
   Local<Array> ret = Array::New(isolate, 2);
 
   int state = promise->State();
-  ret->Set(0, Integer::New(isolate, state));
+  ret->Set(env->context(), 0, Integer::New(isolate, state)).FromJust();
   if (state != Promise::PromiseState::kPending)
-    ret->Set(1, promise->Result());
+    ret->Set(env->context(), 1, promise->Result()).FromJust();
 
   args.GetReturnValue().Set(ret);
 }
 
 static void GetProxyDetails(const FunctionCallbackInfo<Value>& args) {
+  Environment* env = Environment::GetCurrent(args);
   // Return undefined if it's not a proxy.
   if (!args[0]->IsProxy())
     return;
@@ -76,8 +83,8 @@ static void GetProxyDetails(const FunctionCallbackInfo<Value>& args) {
   Local<Proxy> proxy = args[0].As<Proxy>();
 
   Local<Array> ret = Array::New(args.GetIsolate(), 2);
-  ret->Set(0, proxy->GetTarget());
-  ret->Set(1, proxy->GetHandler());
+  ret->Set(env->context(), 0, proxy->GetTarget()).FromJust();
+  ret->Set(env->context(), 1, proxy->GetHandler()).FromJust();
 
   args.GetReturnValue().Set(ret);
 }
@@ -96,7 +103,7 @@ static void PreviewEntries(const FunctionCallbackInfo<Value>& args) {
     return args.GetReturnValue().Set(entries);
   Local<Array> ret = Array::New(env->isolate(), 2);
   ret->Set(env->context(), 0, entries).FromJust();
-  ret->Set(env->context(), 1, v8::Boolean::New(env->isolate(), is_key_value))
+  ret->Set(env->context(), 1, Boolean::New(env->isolate(), is_key_value))
       .FromJust();
   return args.GetReturnValue().Set(ret);
 }
@@ -171,7 +178,7 @@ void SafeGetenv(const FunctionCallbackInfo<Value>& args) {
   args.GetReturnValue()
       .Set(String::NewFromUtf8(
             args.GetIsolate(), text.c_str(),
-            v8::NewStringType::kNormal).ToLocalChecked());
+            NewStringType::kNormal).ToLocalChecked());
 }
 
 void EnqueueMicrotask(const FunctionCallbackInfo<Value>& args) {
@@ -198,12 +205,6 @@ void Initialize(Local<Object> target,
   }
 #undef V
 
-  target->DefineOwnProperty(
-    env->context(),
-    OneByteString(env->isolate(), "pushValToArrayMax"),
-    Integer::NewFromUnsigned(env->isolate(), NODE_PUSH_VAL_TO_ARRAY_MAX),
-    v8::ReadOnly).FromJust();
-
 #define V(name)                                                               \
   target->Set(context,                                                        \
               FIXED_ONE_BYTE_STRING(env->isolate(), #name),                   \
