@@ -492,7 +492,9 @@ void InitWorker(Local<Object> target,
     Local<String> workerString =
         FIXED_ONE_BYTE_STRING(env->isolate(), "Worker");
     w->SetClassName(workerString);
-    target->Set(workerString, w->GetFunction(env->context()).ToLocalChecked());
+    target->Set(env->context(),
+                workerString,
+                w->GetFunction(env->context()).ToLocalChecked()).FromJust();
   }
 
   env->SetMethod(target, "getEnvMessagePort", GetEnvMessagePort);
