@@ -11,9 +11,10 @@ const path = require('path');
 
 // piping should work as expected with createWriteStream
 
-common.refreshTmpDir();
-const loc = fixtures.path('url-tests.js');
-const fn = path.join(common.tmpDir, 'http2-url-tests.js');
+const tmpdir = require('../common/tmpdir');
+tmpdir.refresh();
+const loc = fixtures.path('person-large.jpg');
+const fn = path.join(tmpdir.path, 'http2-url-tests.js');
 
 const server = http2.createServer();
 
