@@ -1,15 +1,10 @@
-// Flags: --expose-internals
 'use strict';
+
 const assert = require('assert');
 const common = require('../common');
 const http = require('http');
 const net = require('net');
-const MAX = +(process.argv[2] || 8 * 1024); // Command line option, or 8KB.
-
-const { getOptionValue } = require('internal/options');
-
-console.log('pid is', process.pid);
-console.log('max header size is', getOptionValue('--max-http-header-size'));
+const MAX = 8 * 1024; // 8KB
 
 // Verify that we cannot receive more than 8KB of headers.
 
@@ -30,18 +25,30 @@ function finished(client, callback) {
 }
 
 function fillHeaders(headers, currentSize, valid = false) {
-  headers += 'a'.repeat(MAX - headers.length - 3);
+  // llhttp counts actual header name/value sizes, excluding the whitespace and
+  // stripped chars.
+  if (process.versions.hasOwnProperty('llhttp')) {
+    // OK, Content-Length, 0, X-CRASH, aaa...
+    headers += 'a'.repeat(MAX - currentSize);
+  } else {
+    headers += 'a'.repeat(MAX - headers.length - 3);
+  }
+
   // Generate valid headers
   if (valid) {
-    // TODO(mcollina): understand why -32 is needed instead of -1
-    headers = headers.slice(0, -32);
+    // TODO(mcollina): understand why -9 is needed instead of -1
+    headers = headers.slice(0, -9);
   }
   return headers + '\r\n\r\n';
 }
 
+const timeout = common.platformTimeout(10);
+
 function writeHeaders(socket, headers) {
   const array = [];
-  const chunkSize = 100;
+
+  // this is off from 1024 so that \r\n does not get split
+  const chunkSize = 1000;
   let last = 0;
 
   for (let i = 0; i < headers.length / chunkSize; i++) {
@@ -56,25 +63,19 @@ function writeHeaders(socket, headers) {
   next();
 
   function next() {
-    if (socket.destroyed) {
-      console.log('socket was destroyed early, data left to write:',
-                  array.join('').length);
-      return;
-    }
-
-    const chunk = array.shift();
-
-    if (chunk) {
-      console.log('writing chunk of size', chunk.length);
-      socket.write(chunk, next);
+    if (socket.write(array.shift())) {
+      if (array.length === 0) {
+        socket.end();
+      } else {
+        setTimeout(next, timeout);
+      }
     } else {
-      socket.end();
+      socket.once('drain', next);
     }
   }
 }
 
 function test1() {
-  console.log('test1');
   let headers =
     'HTTP/1.1 200 OK\r\n' +
     'Content-Length: 0\r\n' +
@@ -89,28 +90,25 @@ function test1() {
       writeHeaders(sock, headers);
       sock.resume();
     });
-
-    // The socket might error but that's ok
-    sock.on('error', () => {});
   });
 
   server.listen(0, common.mustCall(() => {
     const port = server.address().port;
-    const client = http.get({ port: port }, common.mustNotCall());
+    const client = http.get({ port: port }, common.mustNotCall(() => {}));
 
     client.on('error', common.mustCall((err) => {
       assert.strictEqual(err.code, 'HPE_HEADER_OVERFLOW');
-      server.close(test2);
+      server.close();
+      setImmediate(test2);
     }));
   }));
 }
 
 const test2 = common.mustCall(() => {
-  console.log('test2');
   let headers =
     'GET / HTTP/1.1\r\n' +
     'Host: localhost\r\n' +
-    'Agent: nod2\r\n' +
+    'Agent: node\r\n' +
     'X-CRASH: ';
 
   // /, Host, localhost, Agent, node, X-CRASH, a...
@@ -119,7 +117,7 @@ const test2 = common.mustCall(() => {
 
   const server = http.createServer(common.mustNotCall());
 
-  server.once('clientError', common.mustCall((err) => {
+  server.on('clientError', common.mustCall((err) => {
     assert.strictEqual(err.code, 'HPE_HEADER_OVERFLOW');
   }));
 
@@ -131,46 +129,34 @@ const test2 = common.mustCall(() => {
     });
 
     finished(client, common.mustCall((err) => {
-      server.close(test3);
+      server.close();
+      setImmediate(test3);
     }));
   }));
 });
 
 const test3 = common.mustCall(() => {
-  console.log('test3');
   let headers =
     'GET / HTTP/1.1\r\n' +
     'Host: localhost\r\n' +
-    'Agent: nod3\r\n' +
+    'Agent: node\r\n' +
     'X-CRASH: ';
 
   // /, Host, localhost, Agent, node, X-CRASH, a...
   const currentSize = 1 + 4 + 9 + 5 + 4 + 7;
   headers = fillHeaders(headers, currentSize, true);
 
-  console.log('writing', headers.length);
-
   const server = http.createServer(common.mustCall((req, res) => {
-    res.end('hello from test3 server');
-    server.close();
+    res.end('hello world');
+    setImmediate(server.close.bind(server));
   }));
 
-  server.on('clientError', (err) => {
-    console.log(err.code);
-    if (err.code === 'HPE_HEADER_OVERFLOW') {
-      console.log(err.rawPacket.toString('hex'));
-    }
-  });
-  server.on('clientError', common.mustNotCall());
-
   server.listen(0, common.mustCall(() => {
     const client = net.connect(server.address().port);
     client.on('connect', () => {
       writeHeaders(client, headers);
       client.resume();
     });
-
-    client.pipe(process.stdout);
   }));
 });
 
